/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api.artifact;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipOutputStream;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.admin.api.artifact.exception.InvalidZipArchiveException;
import org.ow2.petals.admin.api.artifact.exception.JBIDescriptorMissingException;

public class ArtifactUtilsTest extends AbstractArtifactTest {

    protected final static String BC_JBI = "bc.jbi.xml";

    protected final static String SE_JBI = "se.jbi.xml";

    protected final static String SA_JBI = "sa.jbi.xml";

    protected final static String SL_JBI = "sl.jbi.xml";

    @Rule
    final public TemporaryFolder temporaryFolder = new TemporaryFolder();

    @Test
    public void createArtifactTest() throws ArtifactException, IOException, URISyntaxException {

        Artifact artifact = ArtifactUtils.createArtifact(this.createArtifactURL(BC_JBI));
        assertTrue(artifact instanceof Component);
        assertTrue(artifact instanceof ArtifactState);

        artifact = ArtifactUtils.createArtifact(this.createArtifactURL(SE_JBI));
        assertTrue(artifact instanceof Component);
        assertTrue(artifact instanceof ArtifactState);

        artifact = ArtifactUtils.createArtifact(this.createArtifactURL(SA_JBI));
        assertTrue(artifact instanceof ServiceAssembly);
        assertTrue(artifact instanceof ArtifactState);

        artifact = ArtifactUtils.createArtifact(this.createArtifactURL(SL_JBI));
        assertTrue(artifact instanceof SharedLibrary);
    }

    @Test
    public void createComponentWithGivenPropertiesTest() throws ArtifactException, IOException,
            URISyntaxException {
        Properties expectedProperties = new Properties();
        expectedProperties.put("key1", "value1");
        expectedProperties.put("key2", "value2");

        Component component = (Component) ArtifactUtils.createArtifact(this.createArtifactURL(BC_JBI),
                expectedProperties);
        assertEquals(expectedProperties, component.getParameters());

        URL propertiesFile = Thread.currentThread().getContextClassLoader()
                .getResource("test.properties");
        component = (Component) ArtifactUtils.createArtifact(this.createArtifactURL(BC_JBI),
                propertiesFile);
        assertEquals(expectedProperties, component.getParameters());
    }

    /**
     * <p>
     * Try to create an {@link Artifact} from an invalid ZIP archive where the JBI descriptor is missing.
     * </p>
     * <p>
     * Expected result: the error {@link InvalidZipArchiveException} is thrown.
     * </p>
     */
    @Test(expected = JBIDescriptorMissingException.class)
    public void createArtifactFromArchiveMissingJbiDescriptor() throws ZipException, IOException, ArtifactException {

        // Create the invalid ZIP archive
        final File zipArchiveFile = this.temporaryFolder.newFile("zip-archive");
        final ZipOutputStream zipArchive = new ZipOutputStream(new FileOutputStream(zipArchiveFile));
        zipArchive.putNextEntry(new ZipEntry("dummy.txt"));
        zipArchive.write("dummy".getBytes());
        zipArchive.closeEntry();
        zipArchive.close();

        try {
            ArtifactUtils.createArtifact(zipArchiveFile.toURI().toURL());
        } catch (final JBIDescriptorMissingException e) {
            assertEquals("Unexpected ZIP archive", zipArchiveFile.toURI().toURL(), e.getInvalidZipArchive());
            throw e;
        }

    }
}
